################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################


from zope.interface import implementsOnly, Interface
from zope.component import adapts, getMultiAdapter

from zope.publisher.interfaces.browser import IBrowserView
from zope.publisher.interfaces.browser import IDefaultBrowserLayer

from plone.app.portlets.manager import ColumnPortletManagerRenderer
from plone.app.portlets.browser.manage import ManageContextualPortlets

from vs.dashboardmanager.interfaces import IPortletPageColumn
from collective.portletpage.browser.interfaces import IManagePortletPagePortletsView

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.memoize.instance import memoize

class PortletPageColumn(ColumnPortletManagerRenderer):
    """Render a column
    """
    
    adapts(Interface, IDefaultBrowserLayer, IBrowserView, IPortletPageColumn)
    template = ViewPageTemplateFile('portletpage-column.pt')
    
    @memoize
    def plone_view(self):
        return getMultiAdapter((self.context, self.request), name=u"plone")
    
    def update(self):
        super(PortletPageColumn, self).update()
        self.ids = set()
    
    def portlet_id(self, portlet):
        plone_view = self.plone_view()
        
        normal_manager = plone_view.normalizeString(self.manager.__name__)
        normal_title = plone_view.normalizeString(portlet['assignment'].title)
        
        while normal_title in self.ids:
            normal_title += '-x'
        self.ids.add(normal_title)

        return 'column-%s-portlet-%s' % (normal_manager, normal_title,)
    
class ManagePortlets(ManageContextualPortlets):
    """View used for the edit screen
    """
    
    implementsOnly(IManagePortletPagePortletsView)
    
    def __init__(self, context, request):
        # Skip past the main parent constructor, since it sets disable_border
        super(ManageContextualPortlets, self).__init__(context, request)
    
    __call__ = ViewPageTemplateFile('portletpage-edit.pt')



################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################

import operator 

from zope.component import getUtility, getMultiAdapter
from zope.app.container.interfaces import INameChooser
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.interfaces import ISiteRoot
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from plone.portlets.interfaces import IPortletManager, IPortletAssignment, IPortletAssignmentMapping
from plone.app.portlets.utils import assignment_mapping_from_key

from vs.dashboardmanager import MF as _
from vs.dashboardmanager.logger import LOG

class AssignmentView(BrowserView):

    template = ViewPageTemplateFile('assignment.pt')

    def getGroups(self):
        """ Return PAS groups """
        search_view = self.context.restrictedTraverse('@@pas_search')
        result = search_view.searchGroups()
        result = [r for r in result if not r['pluginid'] in ('auto_group',)]
        result = sorted(result, key=operator.itemgetter('title')) 
        return result

    def push_assignment(self, userid='', group=''):
        """ Push all assignment portlet page assignment into the
            dashboard of a user or a group of users.
        """

        gt = getToolByName(self.context, 'portal_groups')
        mt = getToolByName(self.context, 'portal_membership')

        if userid and group:
            self.context.plone_utils.addPortalMessage(u'Please specify either a member or a group - but not both', 'error')
            return self.request.response.redirect(self.context.absolute_url() + '/@@assignment')

        if not userid and not group:
            self.context.plone_utils.addPortalMessage(u'Please specify either a member or a group', 'error')
            return self.request.response.redirect(self.context.absolute_url() + '/@@assignment')

        site = getUtility(ISiteRoot)
        if userid:
            userids = [userid]
        else:
            userids = [mt.getMemberById(m).getUserName() for m in gt.getGroupMembers(group)]

        dashboards_updated = dict()
        for userid in userids:

            # iterate over all configured portlet managers (1-4) for PortletPage
            for i in range(1, 5):

                manager_name = 'vs.dashboardmanager.column%d' % i
                manager = getUtility(IPortletManager, name=manager_name, context= self.context)
                mapping = getMultiAdapter((self.context, manager), IPortletAssignmentMapping)

                # get hold of the user dashboard manager
                manager2 = getUtility(IPortletManager, name='plone.dashboard%d' % i)
                mapping2 = assignment_mapping_from_key(site, 
                                                      'plone.dashboard%d' % i, 
                                                       category='user', 
                                                       key=userid, 
                                                       create=True)

                ids2 = [tp[0] for tp in mapping2.items()]

                # and copy over the assignments
                for id, assignment in mapping.items():
                    if not id in ids2:
                        mapping2[id] = assignment
                        LOG.info('assigned portlet %s to %s' % (id, userid))
                        dashboards_updated[userid] = True

        if dashboards_updated:
            self.context.plone_utils.addPortalMessage(u'Dashboard(s) updated for members: %s' 
                                                      % ', '.join(dashboards_updated.keys()))
        else:
            self.context.plone_utils.addPortalMessage(u'No dashboards updated')
        return self.request.response.redirect(self.context.absolute_url())


    def __call__(self, *args, **kw):
        return self.template(*args, **kw)



################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################

from plone.app.portlets.interfaces import IColumn
from plone.portlets.interfaces import IPortletManager

class IPortletPageColumn(IColumn):
    """Marker interface describing columns on a portlet page
    """

class IPortletPageColumn1(IPortletManager):
    """ Portlet manager for vs.dashboardmanager """

class IPortletPageColumn2(IPortletManager):
    """ Portlet manager for vs.dashboardmanager """

class IPortletPageColumn3(IPortletManager):
    """ Portlet manager for vs.dashboardmanager """

class IPortletPageColumn4(IPortletManager):
    """ Portlet manager for vs.dashboardmanager """



################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################


from plone.theme.interfaces import IDefaultPloneLayer


class IThemeSpecific(IDefaultPloneLayer):
    """Marker interface that defines a Zope 3 browser layer.
       If you need to register a viewlet only for the
       "Custom Theme" theme, this interface must be its layer
       (in dashboardmanager/viewlets/configure.zcml).
    """



################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################


import time

from AccessControl import getSecurityManager

from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from vs.dashboardmanager import MF as _

class ManagementView(BrowserView):

    template = ViewPageTemplateFile('management.pt')
    template_folder_id = 'dashboard-templates'

    def available_dashboard_pages(self):
        """ return list of available dashboard templates """

        catalog = getToolByName(self, 'portal_catalog')
        return catalog(portal_type='Portlet Page', 
                       sort_on='getObjPositionInParent')

    def new_dashboard_page(self):
        """ Create a new dashboard page inside the dedicated 
            dashboard templates folder.
        """

        portal_root = self.context.restrictedTraverse('@@plone').context
        if not self.template_folder_id in portal_root.objectIds():
            portal_root.invokeFactory('Folder', id=self.template_folder_id)
            folder = portal_root[self.template_folder_id]
            folder.setTitle(_(u'Dashboard templates'))
            folder.reindexObject()
        else:
            folder = portal_root[self.template_folder_id]

        self.request.response.redirect(folder.absolute_url() + '/createObject?type_name=Portlet+Page')


    def __call__(self, *args, **kw):
        return self.template(*args, **kw)


class PersonalDashboardView(BrowserView):

    template = ViewPageTemplateFile('my-dashboard-pages.pt')

    def my_dashboard_pages(self):

        user = getSecurityManager().getUser()
        roles = set(user.getRoles())
        catalog = getToolByName(self, 'portal_catalog')

        result = list()
        for brain in catalog(portal_type='Portlet Page'):
            page_roles = set(brain.getUsedForGroups)
            if groups.intersection(page_groups):
                result.append(brain)

        return result

    def __call__(self, *args, **kw):
        return self.template(*args, **kw)



################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################


from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

class Base(BrowserView):
    """Base class for index page views
    """

class PortletPageView(Base):
    """Default view for PortletPage """
    
    __call__ = ViewPageTemplateFile('portletpage.pt')



################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################

import config
from cStringIO import StringIO
from Products.CMFCore.utils import getToolByName

class Generator(object):

    def installProducts(self, p, out):
        """QuickInstaller install of required Products"""
        qi = getToolByName(p, 'portal_quickinstaller')
        for product in config.DEPENDENCIES:
            if qi.isProductInstalled(product):
                qi.reinstallProducts([product])
            else:
                qi.installProduct(product, locked=0)
                print >> out, "Product installed: %s \n" %product


def setupVarious(context):

    # Ordinarily, GenericSetup handlers check for the existence of XML files.
    # Here, we are not parsing an XML file, but we use this text file as a
    # flag to check that we actually meant for this import step to be run.
    # The file is found in profiles/default.

    if context.readDataFile('vs.dashboardmanager_various.txt') is None:
        return

    # Add additional setup code here
    out = StringIO()
    site = context.getSite()
    gen = Generator()
    gen.installProducts(site, out)
    logger = context.getLogger(config.PROJECTNAME)
    logger.info(out.getvalue())




################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################


import operator

from zope.component import adapts
from zope.interface import implements, Interface
from archetypes.schemaextender.interfaces import ISchemaExtender
from archetypes.schemaextender.field import ExtensionField

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import LinesField, DisplayList, MultiSelectionWidget
from collective.portletpage.interfaces import IPortletPage


class MyLinesField(ExtensionField, LinesField):
    """ integer field """

class PortletPageExtender(object):

    adapts(IPortletPage)
    implements(ISchemaExtender)

    fields = [MyLinesField('usedForGroups',
                            default=[],
                            vocabulary='getGroups',
                            widget=MultiSelectionWidget(
                                label="Used for groups",
                                label_msgid='label_used_for_groups',
                                i18n_domain='vs.dashboardmanager',
                                ),
                             ),
            ]

    def __init__(self, context):
        self.context = context
    
    def getFields(self):
        return self.fields


def getGroups(self):
    search_view = self.restrictedTraverse('@@pas_search')
    result = search_view.searchGroups()
    result = [r['title'] for r in result if not r['pluginid'] in ('auto_group', )]
    result = sorted(result)
    return DisplayList(zip(result, result))

def getUsedForGroups(self):
    return self.getField('usedForGroups').get(self)

from collective.portletpage.content import PortletPage
PortletPage.getGroups = getGroups
PortletPage.getUsedForGroups = getUsedForGroups




################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################

from plone.portlets.interfaces import IPortletType
from zope.component import adapts, getMultiAdapter, getUtilitiesFor

"""
Monkey-patching PortletManager as workaround for 
http://lists.plone.org/pipermail/product-developers/2010-July/005115.html
"""

def getAddablePortletTypes(self):
    
    addable = []
    for p in getUtilitiesFor(IPortletType):
        #BBB - first condition, because starting with Plone 3.1
        #every p[1].for_ should be a list
        if not isinstance(p[1].for_, list):
            logger.warning("Deprecation Warning Portlet type " + \
              "%s is using a deprecated format for " % p[1].addview + \
              "storing interfaces of portlet managers where it is " + \
              "addable. Its for_ attribute should be a list of portlet " + \
              "manager interfaces, using [zope.interface.Interface] " + \
              "for the portlet type to be addable anywhere. The old " + \
              "format will be unsupported in Plone 4.0.")
            if p[1].for_ is None or p[1].for_.providedBy(self):
                addable.append(p[1])
        else:
#            if [i for i in p[1].for_ if i.providedBy(self)]:
            addable.append(p[1])
    return addable

from plone.portlets.manager import PortletManager
PortletManager.getAddablePortletTypes = getAddablePortletTypes




from setuptools import setup, find_packages
import os

version = '0.1.5'

setup(name='vs.dashboardmanager',
      version=version,
      description="An extended Dashboard manager based on collective.portletpage",
      long_description=open("README.txt").read() + "\n" +
                       open(os.path.join("docs", "HISTORY.txt")).read(),
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='',
      author='Andreas Jung',
      author_email='info@zopyx.com',
      url='http://pypi.python.org/pypi/vs.dashboardmanager',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['vs'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'collective.portletpage',
          'archetypes.schemaextender',
          # -*- Extra requirements: -*-
      ],
      entry_points="""
      # -*- Entry points: -*-
      [z3c.autoinclude.plugin]
      target = plone
      """,
      setup_requires=["PasteScript"],
      paster_plugins=["ZopeSkel"],
      )



################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################


from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone.app.layout.viewlets.common import ViewletBase

# Sample code for a basic viewlet (In order to use it, you'll have to):
# - Un-comment the following useable piece of code (viewlet python class).
# - Rename the viewlet template file ('browser/viewlet.pt') and edit the
#   following python code accordingly.
# - Edit the class and template to make them suit your needs.
# - Make sure your viewlet is correctly registered in 'browser/configure.zcml'.
# - If you need it to appear in a specific order inside its viewlet manager,
#   edit 'profiles/default/viewlets.xml' accordingly.
# - Restart Zope.
# - If you edited any file in 'profiles/default/', reinstall your package.
# - Once you're happy with your viewlet implementation, remove any related
#   (unwanted) inline documentation  ;-p

#class MyViewlet(ViewletBase):
#    render = ViewPageTemplateFile('viewlet.pt')
#
#    def update(self):
#        self.computed_value = 'any output'



################################################################
# vs.dashboardmanager
# (C) 2010, Veit Schiele & Andreas Jung
# Published under the GNU Public Licence V 2 (GPL 2)
################################################################


import patches
from zope.i18nmessageid import MessageFactory

MF = MessageFactory('vs.dashboardmanager')

def initialize(context):
    """Initializer called when used as a Zope 2 product."""

